"""Helper methods for working with S3"""
import os

import attr
import yaml
from aws_conduit.aws import s3

LOCAL_STORE = os.path.join(os.path.expanduser('~'), '.conduit')
if not os.path.exists(LOCAL_STORE):
    os.makedirs(LOCAL_STORE)


@attr.s
class ConduitS3(yaml.YAMLObject):
    """S3 helper class."""
    yaml_loader = yaml.SafeLoader
    yaml_tag = u'!Bucket'
    name = attr.ib()
    region = attr.ib()

    def exists(self):
        """Test if an S3 bucket exists."""
        all_buckets = s3.list_all_buckets()
        return bool(self.name in [bucket['Name'] for bucket in all_buckets])

    def create(self):
        """
        Create an S3 bucket.

        Args:
            name(str): The name of the S3 bucket to create.
            region(str): The region to create the S3 bucket in.
        """
        s3.create_bucket(self.name, self.region)

    def delete(self):
        """Delete an S3 bucket and all of its contents."""
        s3.delete_bucket(self.name)

    def file_exists(self, prefix):
        """
        Test if a particular file exists in an S3 bucket.

        Args:
            prefix(str): The prefix of the file to test for.
            bucket_name(str): The name of the bucket to check in.
        """
        return bool(s3.get_file(self.name, prefix))

    def get_config(self, prefix):
        """
        Get conduit config from S3 and load it from yaml.

        Args:
            prefix(str): The prefix of the yaml config.
        """
        file_name = prefix.split('/')[-1]
        s3.download_file(self.name, prefix, os.path.join(LOCAL_STORE, file_name))
        config = yaml.safe_load(open(os.path.join(LOCAL_STORE, file_name)).read())
        return config

    def put_config(self, content, prefix):
        """
        Put some yaml content into an S3 bucket.

        Args:
            content(dict): An object representnig some yaml configuration.
            prefix(str): The prefix to save the configuration to.
        """
        if isinstance(content, dict):
            file_name = prefix.split('/')[-1]
            print("Uploading {} to {}...".format(file_name, self.name))
            open(os.path.join(LOCAL_STORE, file_name), "w+").write(yaml.dump(content, default_flow_style=False))
        else:
            print("Uploading {} to {}...".format(content, self.name))
            file_name = content
        s3.upload_file(self.name, prefix, os.path.join(LOCAL_STORE, file_name))

    def put_resource(self, path, prefix):
        """
        Put some yaml content into an S3 bucket.

        Args:
            path(dict): An object representnig some yaml configuration.
            prefix(str): The prefix to save the configuration to.
        """
        print("Uploading {} to {}...".format(path, self.name))
        s3.upload_file(self.name, prefix, path)

    def get_url(self):
        """Get the https url for this S3 bucket."""
        return "https://s3.{}.amazonaws.com/{}".format(self.region, self.name)




import json

import boto3
from aws_conduit import helper

IAM = boto3.client('iam')

BASIC_POLICY = {
    'Statement': [
        {
            'Principal': {
                'AWS': helper.get_account_id()
            },
            'Effect': 'Allow',
            'Action': ['sts:AssumeRole']
        }, {
            'Principal': {
                'Service': 'servicecatalog.amazonaws.com'
            },
            'Effect': 'Allow',
            'Action': ['sts:AssumeRole']
        }
    ]
}


def create_role(name, description):
    response = IAM.create_role(
        Path='/conduit/',
        RoleName=name,
        AssumeRolePolicyDocument=json.dumps(BASIC_POLICY),
        Description=description
    )
    return response['Role']


def add_policy(role_name, policy_name):
    IAM.attach_role_policy(
        RoleName=role_name,
        PolicyArn='arn:aws:iam::aws:policy/{}'.format(policy_name)
    )


def list_roles(prefix):
    response = IAM.list_roles(
        PathPrefix=prefix
    )
    return response['Roles']


def put_role_policy(role_name, policy_name, policy):
    IAM.put_role_policy(
        RoleName=role_name,
        PolicyName=policy_name,
        PolicyDocument=json.dumps(policy)
    )



import boto3

SSM = boto3.client('ssm')


def get_param(key, environment):
    response = SSM.get_parameter(
        Name='{}-{}'.format(key, environment),
        WithDecryption=True
    )
    if 'Parameter' in response:
        return response['Parameter']['Value']
    return None



"""Helper methods for working with Service Catalog Portfolios."""
import boto3
import yaml

import attr


@attr.s
class ConduitPortfolio(yaml.YAMLObject):
    """Portfolio helper class."""

    yaml_loader = yaml.SafeLoader
    yaml_tag = u'!Portfolio'
    name = attr.ib()
    provider = attr.ib()
    description = attr.ib(default='No description set')
    portfolio_id = attr.ib(default=None)
    products = attr.ib(default=[])
    service_catalog = boto3.client('servicecatalog')

    def create(self, tags):
        """
        Create a new Service Catalog portfolio.

        Args:
            tags (list): A list of tags to apply to the portfolio
        """
        response = self.service_catalog.create_portfolio(
            DisplayName=self.name,
            Description=self.description,
            ProviderName=self.provider,
            Tags=tags
        )
        self.portfolio_id = response['PortfolioDetail']['Id']

    def delete(self):
        """
        Delete this Service Catalog portfolio.
        """
        portfolio_id = self.get_id()
        print(portfolio_id)
        self.service_catalog.delete_portfolio(
            Id=portfolio_id
        )

    def update(self):
        """
        Update a service catalog protfolio.
        """
        self.service_catalog.update_portfolio(
            Id=self.portfolio_id,
            DisplayName=self.name,
            Description=self.description,
            ProviderName=self.provider
        )

    def exists(self, token=None):
        """
        Test if this portfolio exists.

        Args:
            token (str): (Optional) The NextPageToken returned by boto3.
        """
        portfolios = self._get_portfolio_list(token)
        if self.name in [portfolio['DisplayName'] for portfolio in portfolios['PortfolioDetails']]:
            return True
        elif 'NextPageToken' in portfolios:
            return self.exists(token=portfolios['NextPageToken'])
        return False

    def get_id(self, token=None):
        """
        Get the id of this portfolio.

        Args:
            token (str): (Optional) The NextPageToken returned by boto3.
        """
        if self.portfolio_id:
            return self.portfolio_id
        else:
            portfolios = self._get_portfolio_list(token)
            subset = [dict(
                name=portfolio['DisplayName'],
                id=portfolio['Id']
            ) for portfolio in portfolios['PortfolioDetails']]
            if self.name in [item['name'] for item in subset]:
                self.portfolio_id = next(item['id'] for item in subset if item['name'] == self.name)
                return self.portfolio_id
            elif 'NextPageToken' in portfolios:
                return self.get_id(token=portfolios['NextPageToken'])
            else:
                raise AttributeError('Portfolio was not found')

    def associate_conduit(self, account_id):
        response = self.service_catalog.associate_principal_with_portfolio(
            PortfolioId=self.portfolio_id,
            PrincipalARN='arn:aws:iam::{}:role/conduit/conduit-provisioner-role'.format(account_id),
            PrincipalType='IAM'
        )

    def _get_portfolio_list(self, token=None):
        if token:
            portfolios = self.service_catalog.list_portfolios(
                PageSize=20,
                PageToken=token
            )
        else:
            portfolios = self.service_catalog.list_portfolios(
                PageSize=20
            )
        return portfolios



import attr
import boto3
import yaml
from aws_conduit.aws import iam


@attr.s
class ConduitRole(yaml.YAMLObject):
    yaml_loader = yaml.SafeLoader
    yaml_tag = u'!Role'

    name = attr.ib()
    role_id = attr.ib(default=None)
    role_arn = attr.ib(default=None)

    iam = boto3.client('iam')

    def create(self):
        self._find_role()
        if self.role_arn is None:
            print('Creating role: {}'.format(self.name))
            role = iam.create_role(self.name, 'Conduit deploy role.')
            self.role_id = role['RoleId']
            self.role_arn = role['Arn']

    def update_policy(self, policy):
        print('Updating policy for IAM role: {}'.format(self.name))
        iam.put_role_policy(self.name, 'deployer-policy', policy)

    def _find_role(self):
        if self.role_arn is None:
            response = iam.list_roles('/conduit/')
            for role in response:
                if role['RoleName'] == self.name:
                    print('Syncing role...')
                    self.role_id = role['RoleId']
                    self.role_arn = role['Arn']
                    break



import boto3
import semver
from aws_conduit import conduit_factory as factory
from aws_conduit.conduit_portfolio import ConduitPortfolio

SESSION = boto3.session.Session()
IAM = boto3.client('iam')
STS = boto3.client('sts')

CONFIG_PREFIX = 'conduit.yaml'

RESOURCES_KEY = "__resources__"


def get_region():
    region = SESSION.region_name
    return region


def get_account_id():
    account_id = STS.get_caller_identity().get('Account')
    return account_id


def get_alias():
    """
    Get the users account alias.

    Return:
        alias: The first known account alias.
    """
    aliases = IAM.list_account_aliases()
    if aliases and aliases['AccountAliases']:
        return aliases['AccountAliases'][0]


def get_portfolio(config, name=None, portfolio_id=None):
    for portfolio in config['portfolios']:
        if name is not None:
            if portfolio.name == name:
                return portfolio
        if portfolio_id is not None:
            if portfolio.portfolio_id == portfolio_id:
                return portfolio
    raise ValueError('Portfolio not found: {} {}'.format(portfolio_id, name))


def get_product(config, name=None, product_id=None):
    for portfolio in config['portfolios']:
        for product in portfolio.products:
            if product_id is not None:
                if product.product_id == product_id:
                    return product
            if name is not None:
                if product.name == name:
                    return product
    raise ValueError('Product not found: {} {}'.format(product_id, name))


ACCOUNT_ID = get_account_id()


def inject_config(function):

    start = factory.start()
    bucket = start.create_s3()
    configuration = bucket.get_config(CONFIG_PREFIX)

    def wrapper(*args, **kwargs):
        result = function(*args, **kwargs, config=configuration)
        bucket.put_config(configuration, CONFIG_PREFIX)
        return result

    return wrapper


def find_build_product(spec, config):
    portfolio = None
    product = None
    for port in config['portfolios']:
        if isinstance(port, ConduitPortfolio):
            if port.name == spec['portfolio']:
                portfolio = port
                for prod in portfolio.products:
                    if prod.name == spec['product']:
                        product = prod
                        break
        else:
            if port['name'] == spec['portfolio']:
                portfolio = port
                for prod in portfolio['products']:
                    if prod['name'] == spec['product']:
                        product = prod
                        break

    return dict(
        product=product,
        portfolio=portfolio
    )


def get_all_portfolio_artifacts(portfolio_name, config):
    templates = []
    for port in config['portfolios']:
        if isinstance(port, ConduitPortfolio):
            if port.name == portfolio_name:
                for product in port.products:
                    templates.append(dict(
                        template=product.template_location,
                        product=product.name
                    ))
        else:
            if port['name'] == portfolio_name:
                for product in port['products']:
                    templates.append(dict(
                        template=product['template_location'],
                        product=product['name'],
                        version=product['nextVersion'],
                        policy=product['policy']
                    ))
    return templates


def find_provisioned_build_product(to_find_product, spec, config):
    portfolio = None
    product = None
    for port in config['portfolios']:
        if port.name == spec['portfolio']:
            portfolio = port
            for prod in portfolio.products:
                if hasattr(prod, provisioned):
                    for prov in prod.provisioned:
                        if prov == to_find_product:
                            product = prod
                            break

    if portfolio is None or not portfolio.exists():
        raise ValueError("The specified portfolio does not exist: {}".format(to_find_product))
    elif product is None or not product.exists():
        raise ValueError("The product {} does not exist in portfolio {}".format(to_find_product, spec['portfolio']))
    return product


def find_s3_build_product(spec, config):
    default_product = dict(
        name=spec['product'],
        currentVersion='0.0.0',
        nextVersion='0.0.0'
    )
    result = find_build_product(spec, config)
    if result['portfolio'] is None:
        result['portfolio'] = dict(
            name=spec['portfolio'],
            products=[default_product]
        )
        result['product'] = default_product
        config['portfolios'].append(result['portfolio'])
    elif result['product'] is None:
        result['product'] = default_product
        result['portfolio']['products'].append(default_product)

    return result


def next_version(release_type, current_version):
    product_version = current_version
    if release_type == 'build':
        product_version = semver.bump_build(current_version)
    if release_type == 'major':
        product_version = semver.bump_major(current_version)
    if release_type == 'minor':
        product_version = semver.bump_minor(current_version)
    if release_type == 'patch':
        product_version = semver.bump_patch(current_version)
    return product_version


def put_resource(path, bucket, portfolio, product, version, environment='core'):
    if environment is not None:
        key = "{}/{}/{}/{}/{}".format(portfolio, product, environment, version, path)
        directory = "{}/{}/{}/{}/{}".format(bucket.name, portfolio, product, environment, version)
    else:
        key = "{}/{}/{}/{}".format(portfolio, product, version, path)
        directory = "{}/{}/{}/{}".format(bucket.name, portfolio, product, version)
    print("Adding resource to release: {}".format(path))
    replace_resources(directory, path=path)
    bucket.put_resource(path, key)
    revert_resources(directory, path=path)
    return "https://s3-{}.amazonaws.com/{}/{}".format(get_region(), directory, path)


def read_write(function):

    def wrapper(*args, **kwargs):
        if 'path' in kwargs:
            if kwargs['path'].endswith('yaml') or kwargs['path'].endswith('yml') or kwargs['path'].endswith('json'):
                f = open(kwargs['path'], 'r')
                filedata = f.read()
                f.close()
                newdata = function(*args, **kwargs, file_data=filedata)
                if newdata is not None:
                    f = open(kwargs['path'], 'w')
                    f.write(newdata)
                    f.close()
    return wrapper


@read_write
def replace_resources(directory, path=None, file_data=None):
    if file_data is not None:
        print("Replacing in {}".format(path))
        return file_data.replace(RESOURCES_KEY, directory)


@read_write
def revert_resources(directory, path=None, file_data=None):
    if file_data is not None:
        print("Replacing in {}".format(path))
        return file_data.replace(directory, RESOURCES_KEY)


def put_sls_resource(path, bucket, portfolio, product, version, sls_package, environment='core'):
    new_path = path
    if '.serverless' in new_path:
        new_path = new_path.replace('.serverless/', '')
    directory = "{}/{}/{}/{}".format(portfolio, product, environment, version)
    key = "{}/{}/{}/{}/{}".format(portfolio, product, environment, version, new_path)
    replace_sls_resources(directory, bucket.name, sls_package, environment, path=path)
    print("Adding sls resource to release: {}".format(path))
    bucket.put_resource(path, key)
    revert_sls_resources(directory, bucket.name, sls_package, environment, path=path)
    return "https://s3-{}.amazonaws.com/{}/{}/{}".format(get_region(), bucket.name, directory, new_path)


@read_write
def replace_sls_resources(key, bucket, sls_package, environment, path=None, file_data=None):
    if file_data is not None:
        print("Replacing in {}".format(path))
        print("The key is: {}".format(key))
        return file_data.replace(sls_package['artifactDirectoryName'], key).replace(sls_package['bucket'], bucket).replace('${STAGE}', environment)


@read_write
def revert_sls_resources(key, bucket, sls_package, environment, path=None, file_data=None):
    if file_data is not None:
        print("Reverting in {}".format(path))
        print("The key is: {}".format(key))
        return file_data.replace(key, sls_package['artifactDirectoryName']).replace(bucket, sls_package['bucket']).replace(environment, '${STAGE}')



import json

import boto3

SERVICE_CATALOG = boto3.client('servicecatalog')
ROW_FORMAT = "{:<30}" * 3


def associate(product_id, portfolio_id):
    SERVICE_CATALOG.associate_product_with_portfolio(
        ProductId=product_id,
        PortfolioId=portfolio_id
    )


def disassociate(product_id, portfolio):
    SERVICE_CATALOG.disassociate_product_from_portfolio(
        ProductId=product_id,
        PortfolioId=portfolio
    )


def create_constraint(portfolio_id, product_id, params, name):
    SERVICE_CATALOG.create_constraint(
        PortfolioId=portfolio_id,
        ProductId=product_id,
        Parameters=json.dumps(params),
        Type='LAUNCH',
        Description='Launch configuration for {}'.format(name)
    )


def create_product(product, support_email, support_url, support_description, tags, template_location):
    create_response = SERVICE_CATALOG.create_product(
        Name=product.name,
        Owner=product.owner,
        Description=product.description,
        Distributor=product.owner,
        SupportDescription=support_description,
        SupportEmail=support_email,
        SupportUrl=support_url,
        ProductType='CLOUD_FORMATION_TEMPLATE',
        Tags=tags,
        ProvisioningArtifactParameters={
            'Name': '0.0.0',
            'Description': 'Initial product creation.',
            'Info': {
                'LoadTemplateFromURL': template_location
            },
            'Type': 'CLOUD_FORMATION_TEMPLATE'
        },
    )
    return create_response


def list_all_portfolios(token=None):
    if token is not None:
        response = SERVICE_CATALOG.list_portfolios(
            PageToken=token
        )
    else:
        response = SERVICE_CATALOG.list_portfolios()
    for portfolio in response['PortfolioDetails']:
        print(ROW_FORMAT.format(portfolio['DisplayName'], portfolio['Id'], portfolio['Description']))
    if 'NextPageToken' in response:
        list_all_portfolios(token=response['NextPageToken'])


def list_portfolios_for_product(product_id, token=None):
    if token:
        response = SERVICE_CATALOG.list_portfolios_for_product(
            ProductId=product_id,
            PageToken=token,
        )
    else:
        response = SERVICE_CATALOG.list_portfolios_for_product(
            ProductId=product_id
        )
    portfolios = [item['Id'] for item in response['PortfolioDetails']]
    if 'NextPageToken' in response:
        portfolios = portfolios + list_portfolios_for_product(product_id, token=response['NextPageToken'])
    return portfolios


def list_all_products(token=None):
    if token is not None:
        response = SERVICE_CATALOG.search_products_as_admin(
            PageToken=token
        )
    else:
        response = SERVICE_CATALOG.search_products_as_admin()
    for product in response['ProductViewDetails']:
        summary = product['ProductViewSummary']
        print(ROW_FORMAT.format(summary['Name'], summary['ProductId'], summary['ShortDescription']))
    if 'NextPageToken' in response:
        list_all_products(token=response['NextPageToken'])


def list_product_constraints(portfolio_id, product_id):
    response = SERVICE_CATALOG.list_constraints_for_portfolio(
        PortfolioId=portfolio_id,
        ProductId=product_id
    )
    return response['ConstraintDetails']


def get_provisioning_parameters(product_id, version_id, launch_path):
    response = SERVICE_CATALOG.describe_provisioning_parameters(
        ProductId=product_id,
        ProvisioningArtifactId=version_id,
        PathId=launch_path
    )
    return response['ProvisioningArtifactParameters']


def get_all_launch_paths(product_id):
    launch_paths = SERVICE_CATALOG.list_launch_paths(
        ProductId=product_id,
    )
    return launch_paths['LaunchPathSummaries']


def search(term):
    response = SERVICE_CATALOG.search_products_as_admin(
        Filters={
            'FullTextSearch': [
                term,
            ]
        }
    )
    return response


def update_product(product_id, name, owner, description, support_description, support_url, support_email):
    SERVICE_CATALOG.update_product(
        Id=product_id,
        Name=name,
        Owner=owner,
        Description=description,
        Distributor=owner,
        SupportDescription=support_description,
        SupportEmail=support_email,
        SupportUrl=support_url,
    )


def delete_version(product_id, version_id):
    SERVICE_CATALOG.delete_provisioning_artifact(
        ProductId=product_id,
        ProvisioningArtifactId=version_id
    )


def list_all_versions(product_id):
    response = SERVICE_CATALOG.list_provisioning_artifacts(
        ProductId=product_id
    )
    return response['ProvisioningArtifactDetails']


def new_version(product_id, name, template_url):
    print(template_url)
    description = 'Release Candidate build increment'
    if 'build' in name:
        description = 'Incremental build; Not production ready!'
    SERVICE_CATALOG.create_provisioning_artifact(
        ProductId=product_id,
        Parameters={
            'Name': name,
            'Description': description,
            'Info': {
                'LoadTemplateFromURL': template_url
            },
            'Type': 'CLOUD_FORMATION_TEMPLATE'
        }
    )


def provision(client, product, name, params):
    client.provision_product(
        ProductId=product.product_id,
        ProvisioningArtifactId=product.get_version_id(),
        ProvisionedProductName=name,
        ProvisioningParameters=params
    )


def update_provisioned(client, product, name, params):
    client.update_provisioned_product(
        ProvisionedProductName=name,
        ProductId=product.product_id,
        ProvisioningArtifactId=product.get_version_id(),
        ProvisioningParameters=params
    )


def terminate_provisioned(client, name):
    client.terminate_provisioned_product(
        ProvisionedProductName=name,
        IgnoreErrors=False
    )


def is_provisioned(name, token=None):
    if token:
        response = SERVICE_CATALOG.scan_provisioned_products(
            PageToken=token
        )
    else:
        response = SERVICE_CATALOG.scan_provisioned_products(
            AccessLevelFilter={
                'Key': 'Account',
                'Value': 'self'
            }
        )
    for product in response['ProvisionedProducts']:
        print(product)
        if product['Name'] == name:
            return True
    if 'NextPageToken' in response:
        return is_provisioned(name, token=response['NextPageToken'])
    return False



import boto3

from aws_conduit import helper
from aws_conduit.conduit_portfolio import ConduitPortfolio
from aws_conduit.conduit_product import ConduitProduct
from aws_conduit.conduit_role import ConduitRole
from aws_conduit.conduit_s3 import ConduitS3
from aws_conduit.conduit_start import ConduitStart

SESSION = boto3.session.Session()
REGION = SESSION.region_name


def start():
    account_id = helper.ACCOUNT_ID
    return ConduitStart(account_id)


def s3(name):
    """
    Creates a new handle for an S3 bucket.

    Args:
        name (str): The name of the S3 bucket.

    Return:
        ConduitS3 (obj): An unpersisted instance of an S3 bucket.
    """
    return ConduitS3(name, REGION)


def portfolio(portfolio_name, portfolio_description=None):
    """
    Creates a new handle for a Service Catalog portfolio.

    Args:
        portfolio_name (str): The name to aply to the Portfolio.
        portfolio_provider (str): The name of the Portfolio provider.
        portfolio_description (str): Information about the Portfolio.

    Return:
        ConduitPortfolio: An unpersistened instance of a Portfolio.
    """
    portfolio_provider = helper.get_alias()
    return ConduitPortfolio(
        name=portfolio_name,
        description=portfolio_description,
        provider=portfolio_provider
    )


def product_by_id(product_id, token=None):
    client = boto3.client('servicecatalog')
    if token is not None:
        response = client.search_products_as_admin(
            PageToken=token
        )
    else:
        response = client.search_products_as_admin()
    if response['ProductViewDetails']:
        for found_product in response['ProductViewDetails']:
            if found_product['ProductViewSummary']['ProductId'] == product_id:
                print("Creating instance of Conduit handle...")
                summary = found_product['ProductViewSummary']
                conduit_product = product(summary['Name'], None, summary['ShortDescription'])
                conduit_product.product_id = product_id
                return conduit_product
    if 'NextPageToken' in response:
        return product_by_id(product_id, token=response['NextPageToken'])


def product(product_name, portfolio_name, product_description=None):
    """
    Creates a new handle for a Service Catalog Product.

    Args:
        product_name (str): The name of the Product.
        product_owner (str): The owner of the product.
        s3_bucket (obj): A Conduit handle on an S3 Bucket.
        cfntype (str): yaml or json.
        portfolio_name (str): The Portfolio which this Product can be applied to.
        product_description (str): Information about the product.

    Return:
        ConduitProduct: An unpersisted instance of a Product.
    """
    product_owner = helper.get_alias()
    s3_bucket = start().create_s3()
    cfntype = "yaml"
    return ConduitProduct(
        name=product_name,
        owner=product_owner,
        bucket=s3_bucket,
        cfn_type=cfntype,
        portfolio=portfolio_name,
        description=product_description
    )


def role(role_name):
    role = ConduitRole(role_name)
    return role



from datetime import datetime

import attr
from aws_conduit import conduit_factory as factory
from aws_conduit.aws import iam

BUCKET_PREFIX = 'conduit-config-'
CONFIG_PREFIX = 'conduit.yaml'


@attr.s
class ConduitStart(object):

    account_id = attr.ib()

    def create_s3(self):
        bucket_name = BUCKET_PREFIX + self.account_id
        bucket = factory.s3(bucket_name)
        if not bucket.exists():
            print("Creating S3: " + bucket_name)
            bucket.create()
            print("Setting initial configuration...")
            bucket.put_config(dict(created=datetime.now()), CONFIG_PREFIX)
        return bucket

    def create_iam_role(self):
        response = iam.list_roles('/conduit/')
        if not response:
            print('Setting up IAM role...')
            iam.add_policy('conduit-provisioner-role', 'ServiceCatalogAdminFullAccess')
            iam.add_policy('conduit-provisioner-role', 'ServiceCatalogEndUserAccess')



"""A conduit for CI Pipelines in AWS!"""
import json
import os
import subprocess

import semver
import yaml
from aws_conduit import conduit_factory as factory
from aws_conduit import conduit_s3, helper
from aws_conduit.aws import cloudformation, iam, s3, service_catalog
from aws_conduit.helper import inject_config

CONFIG_PREFIX = 'conduit.yaml'


def configure():
    """
    Setup necessary resources for running Conduit

    Return:
        bucket: An object handle on the Conduit configuration bucket.
    """
    start = factory.start()
    bucket = start.create_s3()
    start.create_iam_role()
    return bucket


@inject_config
def sync(config=None):
    account_id = helper.get_account_id()
    print("Ensuring Conduit is up to date...")
    if 'portfolios' in config:
        for portfolio in config['portfolios']:
            if portfolio.exists():
                print("Associating conduit with {}".format(portfolio.name))
                portfolio.associate_conduit(account_id)
            else:
                print("Portfolio no longer exists!")
                config['portfolios'].remove(portfolio)


@inject_config
def new_portfolio(name, description, tags=None, config=None):
    """
    Create a new portfolio.

    Args:
        name (str): The name of the portfolio to create.
        description (str): A description of the portfolio to create.
        tags (list): An optional list of tags to apply to all products in the portfolio.

    Return:
        portfolio: An object handle on the new portfolio.
    """
    if name is None or description is None:
        raise ValueError("name and description must have values")
    if tags is None:
        tags = []
    alias = helper.get_alias()
    if alias:
        print("Creating a new portfolio...")
        portfolio = factory.portfolio(name, portfolio_description=description)
        portfolio.create(tags)
        print("Create complete...")
        if 'portfolios' not in config:
            config['portfolios'] = []
        config['portfolios'].append(portfolio)
        return portfolio
    else:
        raise ValueError('An account alias needs to be set!')


@inject_config
def update_portfolio(portfolio_id, name=None, description=None, config=None):
    """
    Update a portfolio.

    Args:
        name (str): The name of the portfolio to create.
        description (str): A description of the portfolio to create.

    Return:-
        portfolio: An object handle on the portfolio.
    """
    if portfolio_id is None:
        raise ValueError("A portfolio ID must be provided")
    print("Updating portfolio with id: {}".format(portfolio_id))
    portfolio = helper.get_portfolio(config, portfolio_id=portfolio_id)
    if name is not None:
        portfolio.name = name
    if description is not None:
        portfolio.description = description
    portfolio.update()
    print("Portfolio updated successfully...")


@inject_config
def delete_portfolio(portfolio_id, config=None):
    """
    Delete a portfolio.

    Args:
        id (str): The id of the portfolio to delete.
    """
    if portfolio_id is None:
        raise ValueError("A portfolio id must be provided")
    print("Deleting portfolio with id: {}".format(portfolio_id))
    portfolio = helper.get_portfolio(config, portfolio_id=portfolio_id)
    for product in portfolio.products:
        print("Disassociating product with id: {}".format(product.product_id))
        product.disassociate(portfolio.portfolio_id)
    portfolio.delete()
    config['portfolios'].remove(portfolio)


def list_portfolios():
    print(service_catalog.ROW_FORMAT.format("Name", "Id", "Description"))
    print("----------" * 9)
    service_catalog.list_all_portfolios()


@inject_config
def new_product(name, description, cfntype, portfolio_name, tags=None, config=None):
    """
    Create a new product.

    Args:
        name (str): The name of the product to create.
        description (str): A brief description of the product.
        cfntype (str): One of [json, yaml, yml]
        portfolio_name (str): The name of the portfolio to add the product to.

    Return:
        product: An object handle on the new product.
    """
    if name is None or description is None or cfntype is None or portfolio_name is None:
        raise ValueError("name, description, cfntype and portfolio_name must have values")
    if tags is None:
        tags = []
    print("Creating a new product...")
    product = factory.product(name, portfolio_name, product_description=description)
    portfolio = factory.portfolio(portfolio_name)
    portfolio_id = portfolio.get_id()
    support = dict() if 'support' not in config else config['support']
    product.create(support, tags)
    print("Product created...")
    product.add_to_portfolio(portfolio_id)
    print("Product assigned to portfolio: {}".format(portfolio_id))
    portfolio = helper.get_portfolio(config, name=portfolio_name)
    portfolio.products.append(product)
    return product


@inject_config
def update_product(product_id, name, description, cfntype, tags=None, config=None):
    """
    Update a portfolio.

    Args:
        name (str): The name of the portfolio to create.
        description (str): A description of the portfolio to create.

    Return:-
        portfolio: An object handle on the portfolio.
    """
    if product_id is None:
        raise ValueError("A product ID must be provided")
    print("Updating product with id: {}".format(product_id))
    product = helper.get_product(config, product_id=product_id)
    if name is not None:
        product.name = name
    if description is not None:
        product.description = description
    if cfntype is not None:
        product.cfntype = cfntype
    if tags is not None:
        product.tags = tags
    support = dict()
    if 'support' in config:
        support = config['support']
    product.update(support)
    print("Product updated successfully...")


@inject_config
def delete_product(product_id, config=None):
    """
    Delete a product.

    Args:
        id (str): The id of the product to delete.
    """
    if product_id is None:
        raise ValueError("A product ID must be provided")
    print("Deleting product with id: {}".format(product_id))
    for portfolio in config['portfolios']:
        for product in portfolio.products:
            if product.product_id == product_id:
                product.delete()
                portfolio.products.remove(product)
                print("Product deleted successfully...")
                break


@inject_config
def associate_product_with_portfolio(product_id, portfolio_id, config=None):
    if portfolio_id is None:
        raise ValueError("A portfolio id must be provided")
    if product_id is None:
        raise ValueError("A product id must be provided")

    portfolio = helper.get_portfolio(config, portfolio_id=portfolio_id)
    if portfolio is None:
        raise ValueError("Provided portfolio does not exist in Conduit config!")

    bucket = configure()
    print("Associating product with portfolio...")
    service_catalog.associate(product_id, portfolio_id)
    print("Association successful...")
    print("Finding product by id...")
    product = factory.product_by_id(product_id, bucket)
    print("Reflecting changes in Conduit config...")
    portfolio.products.append(product)
    product.portfolio = portfolio.name


@inject_config
def terminate_provisioned_product(product_id, stack_name, config=None):
    if product_id is None:
        raise ValueError("A product id must be provided")
    print("Terminating product...")
    product = helper.get_product(config, product_id=product_id)
    product.terminate(stack_name)


@inject_config
def provision_product(product_id, name, config=None):
    if product_id is None:
        raise ValueError("A product id must be provided")
    print("Provisioning product...")
    product = helper.get_product(config, product_id=product_id)
    _provision(product, name)


def list_products():
    print(service_catalog.ROW_FORMAT.format("Name", "Id", "Description"))
    print("----------" * 9)
    service_catalog.list_all_products()


@inject_config
def set_default_support_config(description=None, email=None, url=None, config=None):
    """
    Set the support configuration for your Service Catalog products.

    Args:
        description (str): Support information about the product.
        email (str): Contact email for product support.
        url (str): Contact URL for product support.
    """
    print("Reading current configuration...")
    support = dict()
    if description:
        support['description'] = description
    if email:
        support['email'] = email
    if url:
        support['url'] = url
    config['support'] = support
    print("Writing new support configuration...")


def build(action, product):
    print("Releasing a new build version...")
    spec = yaml.safe_load(open('conduitspec.yaml').read())
    for product_spec in spec['inventory']:
        if (product is not None and product_spec['product'] == product) or product is None:
            # Perform Build Steps
            if 'build' in product_spec:
                for step in product_spec['build']:
                    subprocess.call(step, shell=True)
            if 'serviceCatalog' in product_spec and product_spec['serviceCatalog']:
                _service_catalog_build(action, product_spec)
            else:
                _s3_build(action, product_spec)


@inject_config
def _service_catalog_build(action, product_spec, config=None):
    result = helper.find_build_product(product_spec, config)
    if result['product'] is None:
        raise ValueError('Product was not found in config!')
    product = result['product']
    print(product_spec)
    update_iam_role(product_spec)
    product.add_resources(product_spec)
    product.create_deployer_launch_constraint(helper.get_portfolio(config, name=product_spec['portfolio']), product_spec['roleName'])
    product.release(action, product_spec['artifact'], product.version)
    if action != 'build':
        product.tidy_versions()


@inject_config
def _s3_build(action, product_spec, config=None):
    result = helper.find_s3_build_product(product_spec, config)
    next_version = helper.next_version(action, result['product']['nextVersion'])
    print("The next version is: {}".format(next_version))
    result['product']['nextVersion'] = next_version
    start = factory.start()
    bucket = start.create_s3()
    sls_package = None

    if 'sls' in product_spec and product_spec['sls'] is True:
        sls_state = json.load(open('.serverless/serverless-state.json'))
        sls_package = sls_state['package']
        sls_package['bucket'] = sls_state['service']['provider']['deploymentBucketObject']['name']
        result['product']['template_location'] = helper.put_sls_resource(
            product_spec['artifact'], bucket, product_spec['portfolio'], product_spec['product'], next_version, sls_package)
    else:
        result['product']['template_location'] = helper.put_resource(
            product_spec['artifact'], bucket, product_spec['portfolio'], product_spec['product'], next_version)

    if 'deployProfile' in product_spec:
        statements = []
        policy = dict(
            Version="2012-10-17",
            Statement=statements
        )
        for entry in product_spec['deployProfile']:
            statement = dict(
                Effect="Allow",
                Action=entry['actions'],
                Resource=entry['resources']
            )
            statements.append(statement)
        result['product']['policy'] = policy

    if 'associatedResources' in product_spec:
        _put_resources(product_spec['associatedResources'], product_spec, bucket, next_version, sls_package)
    if 'nestedStacks' in product_spec:
        _put_resources(product_spec['nestedStacks'], product_spec, bucket, next_version, sls_package)

    # if action != 'build':
    #    _tidy_versions(result['portfolio']['name'], result['product']['name'], next_version, bucket)


def _tidy_versions(portfolio, product, version, bucket):
    versions = s3.get_sub_folders(bucket.name, "{}/{}/{}".format(portfolio, product, 'core'))
    for item in versions:
        this_version = item.split("/")[-1]
        if ('+build' in this_version and
                semver.compare(this_version, version) == -1):
            prefix = "{}/{}/{}/{}".format(portfolio, product, 'core', this_version)
            print("Tidying version: {}".format(prefix))
            s3.delete_folder(bucket.name, prefix)


def _put_resources(resources, product_spec, bucket, next_version, sls_package):
    for resource in resources:
        if 'sls' in product_spec and product_spec['sls'] is True:
            helper.put_sls_resource(resource, bucket, product_spec['portfolio'], product_spec['product'], next_version, sls_package)
        else:
            helper.put_resource(resource, bucket, product_spec['portfolio'], product_spec['product'], next_version)


@inject_config
def package_portfolio(portfolio_name, environment, config=None):
    package = []
    results = helper.get_all_portfolio_artifacts(portfolio_name, config)
    for result in results:
        params = cloudformation.list_parameters(result['template'])
        parameters = []
        for param in params:
            if param['ParameterKey'] == 'Environment':
                parameters.append(dict(
                    ParameterKey='Environment',
                    ParameterValue=environment
                ))
            else:
                parameters.append(dict(
                    ParameterKey=param['ParameterKey']
                ))
        package.append(dict(
            template=result['template'],
            version=result['version'],
            parameters=parameters,
            product=result['product'],
            policy=result['policy']
        ))
    print(json.dumps(package))

    start = factory.start()
    bucket = start.create_s3()

    file_name = '{}.json'.format(portfolio_name)
    file_path = os.path.join(conduit_s3.LOCAL_STORE, file_name)
    zip_name = '{}-{}.zip'.format(portfolio_name, environment)
    zip_path = os.path.join(conduit_s3.LOCAL_STORE, zip_name)
    open(file_path, "w+").write(json.dumps(package))
    subprocess.call('cd {} && zip -r {} {}'.format(conduit_s3.LOCAL_STORE, zip_name, file_name), shell=True)
    bucket.put_resource(zip_path, '{}/{}-{}.zip'.format(portfolio_name, portfolio_name, environment))


@inject_config
def package_product(portfolio_name, product_name, environment, config=None):
    package = []
    results = helper.get_all_portfolio_artifacts(portfolio_name, config)
    for result in results:
        if result['product'] == product_name:
            params = cloudformation.list_parameters(result['template'])
            parameters = []
            for param in params:
                if param['ParameterKey'] == 'Environment':
                    parameters.append(dict(
                        ParameterKey='Environment',
                        ParameterValue=environment
                    ))
                else:
                    parameters.append(dict(
                        ParameterKey=param['ParameterKey']
                    ))
            package.append(dict(
                template=result['template'],
                version=result['version'],
                parameters=parameters,
                product=result['product'],
                policy=result['policy']
            ))
    print(json.dumps(package))

    start = factory.start()
    bucket = start.create_s3()

    file_name = '{}-{}.json'.format(portfolio_name, product_name)
    file_path = os.path.join(conduit_s3.LOCAL_STORE, file_name)
    zip_name = '{}-{}-{}.zip'.format(portfolio_name, product_name, environment)
    zip_path = os.path.join(conduit_s3.LOCAL_STORE, zip_name)
    open(file_path, "w+").write(json.dumps(package))
    subprocess.call('cd {} && zip -r {} {}'.format(conduit_s3.LOCAL_STORE, zip_name, file_name), shell=True)
    bucket.put_resource(zip_path, '{}/{}-{}-{}.zip'.format(portfolio_name, portfolio_name, product_name, environment))


@inject_config
def provision_product_build(product_name, name, config=None):
    if name is None:
        raise ValueError("A stage must be provided")
    spec = yaml.safe_load(open('conduitspec.yaml').read())
    if 'products' in spec:
        product_spec = None
        for product in spec['products']:
            if product['name'] == product_name:
                product_spec = product
        if product_spec is None:
            raise ValueError("The requested product was not defined in conduitspec.yaml")
        print("Provisioning product...")
        product = helper.find_build_product(product_name, spec, config)
        update_iam_role(product_spec)
        _provision(product, name)
        if not hasattr(product, 'provisioned'):
            product.provisioned = []
        product.provisioned.append(name)
    else:
        raise ValueError("No products defined in conduitspec.yaml")


def _provision(product, name):
    version_id = product.get_version_id()
    launch_paths = service_catalog.get_all_launch_paths(product.product_id)
    print("Getting launch path...")
    if launch_paths:
        launch_path = launch_paths[0]['Id']
        print("Getting input parameters...")
        provisioning_params = service_catalog.get_provisioning_parameters(product.product_id,
                                                                          version_id,
                                                                          launch_path)
        params = []
        for param in provisioning_params:
            if param['ParameterKey'] != "ConduitStackKey":
                if 'DefaultValue' in param:
                    input_value = input('{} (Default: {}): '.format(param['ParameterKey'],
                                                                    param['DefaultValue']))
                else:
                    input_value = input('{}: '.format(param['ParameterKey']))
                if input_value is None or input_value == '':
                    input_value = param['DefaultValue']
                param = dict(
                    Key=param['ParameterKey'],
                    Value=input_value
                )
                params.append(param)
        print(params)
        product.provision(params, name)


@inject_config
def terminate_product(provisioned_product_name, config=None):
    if provisioned_product_name is None:
        raise ValueError("A product name must be provided")
    spec = yaml.safe_load(open('conduitspec.yaml').read())
    print("Terminating product...")
    product = None
    for port in config['portfolios']:
        if port.name == spec['portfolio']:
            portfolio = port
            for prod in portfolio.products:
                product = prod
                break
    #product = helper.find_provisioned_build_product(provisioned_product_name, spec, config)
    product.terminate(provisioned_product_name)
    product.provisioned.remove(provisioned_product_name)


def update_iam_role(spec):
    try:
        if 'roleName' in spec:
            iam.create_role(spec['roleName'], 'Deployer role for {}'.format(spec['product']))
            iam.add_policy(spec['roleName'], 'ServiceCatalogEndUserFullAccess')
        else:
            raise ValueError('A roleName must be specified for your product.')
    except:
        print('Role probably already exists...')

    if 'deployProfile' in spec:
        print('Updating IAM Role...')
        statements = []
        policy = dict(
            Version="2012-10-17",
            Statement=statements
        )
        for entry in spec['deployProfile']:
            statement = dict(
                Effect="Allow",
                Action=entry['actions'],
                Resource=entry['resources']
            )
            statements.append(statement)

        iam.put_role_policy(spec['roleName'], 'deploy-policy', policy)
        return policy
    else:
        raise ValueError('No deploy profile.  Will not continute...')



import boto3

S3_RESOURCE = boto3.resource('s3')
S3_CLIENT = boto3.client('s3')


def create_bucket(name, region):
    bucket = S3_RESOURCE.Bucket(name)
    bucket.create(
        ACL='private',
        CreateBucketConfiguration={
            'LocationConstraint': region
        }
    )
    bucket.Versioning().enable()


def delete_bucket(name):
    bucket = S3_RESOURCE.Bucket(name)
    bucket.objects.all().delete()
    bucket.delete()


def get_sub_folders(name, prefix):
    sub_folders = []
    bucket = S3_RESOURCE.Bucket(name=name)
    for obj in bucket.objects.filter(Prefix=prefix):
        parts = obj.key.split("/")
        parts.pop(-1)
        folder = '/'.join(str(part) for part in parts)
        sub_folders.append(folder)
    return sub_folders


def delete_folder(name, prefix):
    objects_to_delete = S3_RESOURCE.meta.client.list_objects(Bucket=name, Prefix=prefix)
    delete_keys = {'Objects': []}
    delete_keys['Objects'] = [{'Key': k} for k in [obj['Key'] for obj in objects_to_delete.get('Contents', [])]]
    if delete_keys['Objects']:
        print("Deleting keys: {}".format(delete_keys))
        S3_RESOURCE.meta.client.delete_objects(Bucket=name, Delete=delete_keys)


def get_file(name, prefix):
    bucket = S3_RESOURCE.Bucket(name)
    response = bucket.objects.filter(
        Prefix=prefix
    )
    return response


def download_file(name, prefix, download_location):
    S3_CLIENT.download_file(name, prefix, download_location)


def upload_file(name, prefix, file):
    obj = S3_RESOURCE.Object(name, prefix)
    obj.upload_file(file)


def list_all_buckets():
    all_buckets = S3_CLIENT.list_buckets()
    return all_buckets['Buckets']



from setuptools import setup

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    include_package_data=True,
    name='aws_conduit',
    packages=['aws_conduit', 'aws_conduit.aws'],
    version='0.0.11',
    description='Product management for AWS Service Catalog.',
    author='Connor Bray',
    author_email='connor.bray@icloud.com',
    url="https://github.com/concon121/aws-conduit",
    download_url="https://github.com/concon121/aws-conduit/archive/0.0.11.tar.gz",
    install_requires=required,
    keywords=['aws', 'servicecatalog'],
    entry_points={
        'console_scripts': [
            'conduit=aws_conduit.__init__:main'
        ]
    }
)
